//	-----------------------------------------------------
//
//		Skin:	miniPlayer
//		File:	notifier.m
//		Author:	Aleksandar Ruzicic a.k.a. krckoorascic
//
//		Implements notifier
//
//				http://www.skinconsortium.com
//	-----------------------------------------------------

#include <lib/std.mi>
#include "attribs.m"

Function reset();
Function createNotifier();
Function showNotifier(Int w);
Function onNext();
Function Int max(Int a, Int b);
Function Int properWidth(Int w);

Function Int fillNextTrackInfo(String corneroverride);
Function Int fillCustomInfo(String customstring);
Function Int checkPref(int bypassfs);
Function Int Animation();
Function Int Position();

Global Container notifier_container;
Global Layout notifier_layout;
Global Group notifier_frame;
Global Timer notifier_timer;
Global String last_autotitle;
Global Int cur_status;


// ------------------------------------------------------------------------------
// init
// ------------------------------------------------------------------------------
System.onScriptLoaded()
{

	initAttribs();

	notifier_timer = new Timer;

	if (System.getPrivateInt("miniPlayer", "Notifier loaded", false) == false)
	{
		if (!System.isTransparencyAvailable())
			attNotifierSlideV.setData(ON); //default on non 2k/XP systems

		System.setPrivateInt("miniPlayer", "Notifier loaded", true); //no check in future
	}
}


// ------------------------------------------------------------------------------
// shutdown
// ------------------------------------------------------------------------------
System.onScriptUnloading()
{
	delete notifier_timer;
}


// ------------------------------------------------------------------------------
// called by the system when the global hotkey for notification is pressed
// ------------------------------------------------------------------------------
System.onShowNotification()
{
	if (checkPref(1)) return;

	createNotifier();

	String str = "";

	if (System.getStatus() == STATUS_PLAYING)      str = "Playing";
	else if (System.getStatus() == STATUS_PAUSED)  str = "Playback Paused";
	else if (System.getStatus() == STATUS_STOPPED) str = "Playback Stopped";

	showNotifier(fillNextTrackInfo(str));

	complete;	// prevents other scripts from getting the message

	return 1;	// tells anybody else that might watch the returned value that, yes, we implemented that
}


// ------------------------------------------------------------------------------
// called by the system when the title for the playing item changes, this could be the result of the player
// going to the next track, or of an update in the track title
// ------------------------------------------------------------------------------
System.onTitleChange(String newtitle)
{
	if (last_autotitle == newtitle) return;

	if (System.strLeft(newtitle, 1) == "[")
	{
		if (System.strLower(System.strLeft(newtitle, 7))  == "[buffer" ||
			System.strLower(System.strLeft (newtitle, 4)) == "[icy") return;
	}

	last_autotitle = newtitle;

	onNext();
}


// ------------------------------------------------------------------------------
// called by the system when the user clicks the next button
// ------------------------------------------------------------------------------
onNext()
{
	if (checkPref(0)) return;

	createNotifier();

	showNotifier(fillNextTrackInfo(""));
}


// ------------------------------------------------------------------------------
// called by the system when the user clicks the play button
// ------------------------------------------------------------------------------
System.onPlay()
{
	if (checkPref(0)) return;

	createNotifier();

	showNotifier(fillNextTrackInfo("Playing"));
}


// ------------------------------------------------------------------------------
// called by the system when the user clicks the pause button
// ------------------------------------------------------------------------------
System.onPause()
{
	if (checkPref(0)) return;

	createNotifier();

	showNotifier(fillCustomInfo("Paused"));
}


// ------------------------------------------------------------------------------
// called by the system when the user clicks the pause button again
// ------------------------------------------------------------------------------
System.onResume()
{
	if (checkPref(0)) return;

	createNotifier();

	showNotifier(fillNextTrackInfo("Resuming"));
}


// ------------------------------------------------------------------------------
// called by the system when the user clicks the play button
// ------------------------------------------------------------------------------
System.onStop()
{
	if (checkPref(0)) return;

	createNotifier();

	showNotifier(fillCustomInfo("End of Playback"));
}


// ------------------------------------------------------------------------------
// checks if we should display anything
// ------------------------------------------------------------------------------
Int checkPref(int bypassfs)
{
	if (!bypassfs && attNotifierFullscreen.getData() == ON && System.isVideoFullscreen())
		return 1;

	if (attNotifierNever.getData() == ON)
		return 1;

	if (attNotifierMinimized.getData() == ON && !System.isMinimized())
		return 1;

	if (attNotifierInactive.getData() == ON)
		{
			if (System.isMinimized() || ! System.isAppActive())
				return 0;

			return 1;
		}

	return 0;	//show notifier
}


// ------------------------------------------------------------------------------
// returns animation type [1-fade, 2-slideH, 3-slideV, 0-noanim]
// ------------------------------------------------------------------------------
Int Animation()
{
	if (attNotifierFade.getData() == ON)

		if (isTransparencyAvailable())
			return 1;									// fade in/out

	    else
			return 0;									// no animation

	else
		if (attNotifierSlideH.getData() == ON)
			return 2;									// slide horisontally

		else
			if (attNotifierSlideV.getData() == ON)
				return 3;								// slide vertically

			else
				return 0;								// no animation
}


// ------------------------------------------------------------------------------
// returns position on which we should display notifier
//	 [0-left/top, 1-left/bottom, 2-right/top, 3-right/bottom]
// ------------------------------------------------------------------------------
Int Position()
{
	if (attNotifierLeft.getData() == "1")

		if (attNotifierTop.getData() == "1")
			return 0;                                   // top - left

		else
			return 1;                                   // bottom - left

	else
		if (attNotifierTop.getData() == ON)
			return 2;	                                // top - right

		else
			return 3;									// bottom - right
}


// ------------------------------------------------------------------------------
// fade/slide in/out completed
// ------------------------------------------------------------------------------
notifier_layout.onTargetReached()
{
	if (cur_status == 1)
	{
		notifier_timer.setDelay(System.StringToInteger(attNotifierTimeHold.getData()));
		notifier_timer.start();
	}
	else
		if (cur_status == 0) reset();
}


// ------------------------------------------------------------------------------
// hold time elapsed
// ------------------------------------------------------------------------------
notifier_timer.onTimer()
{
	notifier_timer.stop();

	cur_status = 0; //set close flag

	if (Animation() == 1)					/*		 fade out		*/
		{
			notifier_layout.setTargetA(0);
			notifier_layout.setTargetSpeed(System.stringToInteger(attNotifierTimeOut.getData()) / 1000);
			notifier_layout.gotoTarget();
		}

	else if (Animation() == 2)			/*		slide horisontally	  */
		{
			int x;

			if (Position() == 0 || Position() == 1)
				x = - (System.getViewPortLeft() + notifier_layout.getWidth() + 5);

			else if (Position() == 2 || Position() == 3)
				x = System.getViewPortLeft() + System.getViewPortWidth() + 5;

			notifier_layout.setTargetX(x);
			notifier_layout.setTargetSpeed(System.stringToInteger(attNotifierTimeOut.getData()) / 1000);
			notifier_layout.gotoTarget();
		}

	else if (Animation() == 3)			/*		slide vertically		*/
		{
			int y;

			if (Position() == 0 || Position() == 2)
				y = - (System.getViewPortTop() + 110);

			else if (Position() == 1 || Position() == 3)
				y = System.getViewPortTop() + System.getViewPortHeight() + 110;

			notifier_layout.setTargetY(y);
			notifier_layout.setTargetSpeed(System.stringToInteger(attNotifierTimeOut.getData()) / 1000);
			notifier_layout.gotoTarget();
		}

	else reset();
}


// ------------------------------------------------------------------------------
// when notifier is clicked, bring back the app from minimized state if its minimized and focus it
// ------------------------------------------------------------------------------
notifier_frame.onLeftButtonDown(int x, int y)
{
	notifier_timer.stop();
	notifier_layout.cancelTarget();
	reset();
	System.restoreApplication();
	System.activateApplication();
}


// ------------------------------------------------------------------------------
// when notifier is right-clicked, distroy it
// ------------------------------------------------------------------------------
notifier_frame.onRightButtonUp(int x, int y)
{
	reset();
	complete;
}


// ------------------------------------------------------------------------------
// close the notifier window, destroys the container automatically because it's dynamic
// ------------------------------------------------------------------------------
reset()
{
	notifier_container.close();
	notifier_container = null;
	notifier_layout = null;
}


// ------------------------------------------------------------------------------
createNotifier()
{
	if (notifier_container == null)
	{
		notifier_container = System.newDynamicContainer("notifier");

		if (!notifier_container) return; // reinstall duh!

		notifier_layout = notifier_container.getLayout("normal");

		if (!notifier_layout) return; // reinstall twice, man

		notifier_frame = notifier_layout.findObject("notifier.frame");
	}
	else // notifier_layout != null
	{
		cur_status = 0;
		notifier_layout.cancelTarget();
		notifier_timer.stop();
	}
}


// ------------------------------------------------------------------------------
showNotifier(int w)
{
	cur_status = 1;

	w = properWidth(w + 60);

	int x, y, x0, y0;		// x,y: end position;	 x0,y0: start position

	if (Position() == 0)	/*	 top left  */
	{
		x = System.getViewPortLeft() + 2;
		y = System.getViewPortTop() + 2;

		x0 = x;	y0 = y;

		if (Animation() == 2)
			x0 = System.getViewPortLeft() - w - 10;

		else if (Animation() == 3)
			y0 = - 110;
	}

	else if (Position() == 1)	  /*	 bottom left	*/
	{
		x = System.getViewPortLeft() + 2;
		y = System.getViewPortTop() + System.getViewPortHeight() - 106;

		x0 = x;	y0 = y;

		if (Animation() == 2)
			x0 = System.getViewPortLeft() - w - 5;

		else if (Animation() == 3)
			y0 = System.getViewPortTop() + System.getViewPortHeight() + 110;
	}

	else if (Position() == 2)	  /*	 top right	*/
	{
		x = System.getViewPortLeft() + System.getViewPortWidth() - w - 2;
		y = System.getViewPortTop()  + 2;

		x0 = x;	y0 = y;
		
		if (Animation() == 2)
			x0 = x + w + 10;

		else if (Animation() == 3)
			y0 = - 110;
	}

	else if (Position() == 3)	  /*	 bottom right	*/
	{
		x = System.getViewPortLeft() + System.getViewPortWidth() - w - 2;
		y = System.getViewPortTop()  + System.getViewPortHeight() - 106;

		x0 = x;	y0 = y;

		if (Animation() == 2)
			x0 = x + w + 10;

		else if (Animation() == 3)
			y0 = y + 110;
	}

	if (!notifier_layout.isVisible())
	{
		notifier_layout.resize(x0, y0, w, 104);
		notifier_layout.show();
	}

	if (Animation() == 1)			/*	   fade in		*/
		notifier_layout.setTargetA(255);

	else if (Animation() == 0)		/*	 no animation	*/
	{
		notifier_layout.setAlpha(255);
		notifier_layout.resize(notifier_layout.getLeft(), y, notifier_layout.getWidth(), 104);
		notifier_layout.show();
		notifier_layout.setTargetW(w);
		notifier_layout.setTargetX(x);
		notifier_layout.setTargetSpeed(System.stringToInteger(attNotifierTimeIn.getData()) / 1000);
		notifier_layout.gotoTarget();
		return;
	}

	else notifier_layout.setAlpha(255); //slide v/h

	notifier_layout.setTargetX(x);
	notifier_layout.setTargetY(y);
	notifier_layout.setTargetW(w);
	notifier_layout.setTargetSpeed(System.stringToInteger(attNotifierTimeIn.getData()) / 1000);
	notifier_layout.gotoTarget();
}


// ------------------------------------------------------------------------------
Int fillNextTrackInfo(String corneroverride)
{
	Int maxv = 0;
	Int stream = false;

	Group p = notifier_layout;

	Text plentry = p.findObject("plentry");
	Text nexttrack = p.findObject("nexttrack");
	Text _title = p.findObject("title");
	Text album = p.findObject("album");
	Text artist = p.findObject("artist");
	Text endofplayback = p.findObject("endofplayback");

	if (System.strLower(System.strLeft(System.getPlayItemString(), 7)) == "http://")
		stream = true;

	if (endofplayback) endofplayback.hide();

	if (plentry)
	{ 
		plentry.setText(System.integerToString(System.getPlaylistIndex() + 1) + "/" + 
			            System.integerToString(System.getPlaylistLength()));
		plentry.show();
	}

	if (nexttrack)
	{
		if (corneroverride == "")
		{
			if (!stream)
			{
				if (!System.isVideo())
					nexttrack.setText("New track");
				
				else
					nexttrack.setText("New video");
			}
			else 
				nexttrack.setText("On air");

		}
		else
			nexttrack.setText(corneroverride);

		nexttrack.show();
	}

	if (_title)
	{
		String str;
		
		if (!stream)
		{
			_title.setXmlParam("ticker", "0");
			_title.setXmlParam("display", "");

			str = System.getPlayitemMetaDataString("title");

			if (str == "") str = System.getPlayitemDisplayTitle();

			String l = System.getPlayItemMetaDataString("length");
			
			if (l != "")
			{
				str += " (" + System.integerToTime(System.stringToInteger(l)) + ")";
			}

			_title.setText(str);
		}
		else
		{
			_title.setXmlParam("ticker", "1");
			_title.setXmlParam("display", "SONGTITLE");
			_title.setText("");
		}

		_title.show();
	}

	if (artist)
	{
		if (!stream)
		{
			if (System.isVideo())
				artist.setText("");

			else
				artist.setText(System.getPlayitemMetaDataString("artist"));

		}
		else
		{
			if (System.isVideo())
				artist.setText("Internet TV");

			else
				artist.setText("Internet Radio");
		}
		
		artist.show();
	}

	if (album)
	{
		String str;
		
		if (!stream && !System.isVideo())
		{
			album.setXmlParam("display", "");
			
			str = System.getPlayitemMetaDataString("album");

			String l = System.getPlayitemMetaDataString("track");
			
			if (l != "" && l != "-1") 
				str += " (Track " + l + ")";

			album.setText(str);
		}
		else
		{
			album.setText("");
			album.setXmlParam("display", "SONGINFO");
		}
		
		album.show();
	}

	maxv = max(max(artist.getAutoWidth() + 50, 151),
		       max(_title.getAutoWidth(), album.getAutoWidth()));

	if (maxv > getViewportWidth() / 3)
		maxv = getViewportWidth() / 3;

	return properWidth(maxv);
}

// ------------------------------------------------------------------------------
Int fillCustomInfo(String customtext)
{
	Group p = notifier_layout;

	Text plentry = p.findObject("plentry");
	Text nexttrack = p.findObject("nexttrack");
	Text _title = p.findObject("title");
	Text album = p.findObject("album");
	Text artist = p.findObject("artist");
	Text endofplayback = p.findObject("endofplayback");

	if (plentry) plentry.hide();
	if (nexttrack) nexttrack.hide();
	if (_title) _title.hide();
	if (artist) artist.hide();
	if (album) album.hide();

	if (endofplayback)
	{
		endofplayback.setText(customtext);

		int aw = endofplayback.getAutoWidth();

		endofplayback.show();
		
		if (aw > 151)
			return properWidth(aw);
	}

	return 151;
}

// ------------------------------------------------------------------------------
Int max(Int a, Int b)
{
	if (a > b)
		return a;

	return b;
}


// ------------------------------------------------------------------------------
// make sure that w - 46 is divideable by 3 (cuz middle layer is tiled)
// ------------------------------------------------------------------------------
Int properWidth(Int w)
{
	w -= 46;

	return w + 49 - (w % 3);
}

// ------------------------------------------------------------------------------


// ------------------------------------------------------------------------------
// config attributes stuff...
// ------------------------------------------------------------------------------
//we need this cuz these attributes are radio buttons ;)
#define NOOFF if (getData() == OFF) { setData(ON); return; }

attNotifierAlways.onDataChanged()
{
	if (myChange) return;
	NOOFF
	myChange = true;
	attNotifierInactive.setData(OFF);
	attNotifierMinimized.setData(OFF);
	attNotifierNever.setData(OFF);
	myChange = false;
}

attNotifierMinimized.onDataChanged()
{
	if (myChange) return;
	NOOFF
	myChange = true;
	attNotifierAlways.setData(OFF);
	attNotifierInactive.setData(OFF);
	attNotifierNever.setData(OFF);
	myChange = false;
}

attNotifierNever.onDataChanged()
{
	if (myChange) return;
	NOOFF
	myChange = true;
	attNotifierAlways.setData(OFF);
	attNotifierInactive.setData(OFF);
	attNotifierMinimized.setData(OFF);
	myChange = false;
}

attNotifierInactive.onDataChanged()
{
	if (myChange) return;
	NOOFF
	myChange = true;
	attNotifierAlways.setData(OFF);
	attNotifierMinimized.setData(OFF);
	attNotifierNever.setData(OFF);
	myChange = false;
}

attNotifierLeft.onDataChanged()
{
	if (myChange) return;
	NOOFF
	myChange = true;
	attNotifierRight.setData(OFF);
	myChange = false;
}

attNotifierRight.onDataChanged()
{
	if (myChange) return;
	NOOFF
	myChange = true;
	attNotifierLeft.setData(OFF);
	myChange = false;
}

attNotifierTop.onDataChanged()
{
	if (myChange) return;
	NOOFF
	myChange = true;
	attNotifierBottom.setData(OFF);
	myChange = false;
}

attNotifierBottom.onDataChanged()
{
	if (myChange) return;
	NOOFF
	myChange = true;
	attNotifierTop.setData(OFF);
	myChange = false;
}

attNotifierFade.onDataChanged()
{
	if (myChange) return;
	NOOFF
	myChange = true;
	attNotifierSlideV.setData(OFF);
	attNotifierSlideH.setData(OFF);
	attNotifierNoAnim.setData(OFF);
	myChange = false;
}

attNotifierSlideV.onDataChanged()
{
	if (myChange) return;
	NOOFF
	myChange = true;
	attNotifierFade.setData(OFF);
	attNotifierSlideH.setData(OFF);
	attNotifierNoAnim.setData(OFF);
	myChange = false;
}

attNotifierSlideH.onDataChanged()
{
	if (myChange) return;
	NOOFF
	myChange = true;
	attNotifierFade.setData(OFF);
	attNotifierSlideV.setData(OFF);
	attNotifierNoAnim.setData(OFF);
	myChange = false;
}

attNotifierNoAnim.onDataChanged()
{
	if (myChange) return;
	NOOFF
	myChange = true;
	attNotifierFade.setData(OFF);
	attNotifierSlideV.setData(OFF);
	attNotifierSlideH.setData(OFF);
	myChange = false;
}